@@ -4,7 +4,7 @@
  * GChemPaint library
  * bond.cc 
  *
- * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -103,11 +103,49 @@ bool Bond::GetLine2DCoords (unsigned Num, double* x1, double* y1, double* x2, do
 		double BondDist = Theme->GetBondDist () / Theme->GetZoomFactor ();
 		dx *= (BondDist / l);
 		dy *= (BondDist / l);
+		// now, exclude symbols rectangles from the drawing
+		double ax, ay, anga, angb = atan2 (fabs (dy), fabs (dx));
+		reinterpret_cast <Atom *> (m_Begin)->GetSymbolGeometry (ax, ay, anga, dy < 0);
+		bool horizontal;
+		if (ax > 0) {
+			horizontal = anga >= angb;
+			if (horizontal) {
+				ax = (ax + 2.) / Theme->GetZoomFactor ();
+				if (dx > 0)
+					ax = - ax;
+				*x1 -= ax;
+				*y1 -= ax * dy / dx;
+			} else {
+				ay = (ay + 2.) / Theme->GetZoomFactor ();
+				if (dy > 0)
+					ay = - ay;
+				*y1 -= ay;
+				*x1 -= ay * dx / dy;
+			}
+		}
+		reinterpret_cast <Atom *> (m_End)->GetSymbolGeometry (ax, ay, anga, dy > 0);
+		if (ax > 0) {
+			horizontal = anga >= angb;
+			if (horizontal) {
+				ax = (ax + 2.) / Theme->GetZoomFactor ();
+				if (dx > 0)
+					ax = - ax;
+				*x2 += ax;
+				*y2 += ax * dy / dx;
+			} else {
+				ay = (ay + 2.) / Theme->GetZoomFactor ();
+				if (dy > 0)
+					ay = - ay;
+				*y2 += ay;
+				*x2 += ay * dx / dy;
+			}
+		}
+		// always set the first line coords, even if changed later
+		m_coords[0] = *x1;
+		m_coords[1] = *y1;
+		m_coords[2] = *x2;
+		m_coords[3] = *y2;
 		if (m_order & 1) {
-			m_coords[0] = *x1;
-			m_coords[1] = *y1;
-			m_coords[2] = *x2;
-			m_coords[3] = *y2;
 			if (m_order == 3) {
 				m_coords[4] = *x1 - dy;
 				m_coords[5] = *y1 + dx;
@@ -119,10 +157,6 @@ bool Bond::GetLine2DCoords (unsigned Num, double* x1, double* y1, double* x2, do
 				m_coords[11] = *y2 - dx;
 			}
 		} else if ((m_order == 2) && IsCyclic ()) {
-			m_coords[0] = *x1;
-			m_coords[1] = *y1;
-			m_coords[2] = *x2;
-			m_coords[3] = *y2;
 			Cycle* pCycle;
 			if (IsCyclic() > 1) {
 				//Search prefered cycle
@@ -137,23 +171,145 @@ bool Bond::GetLine2DCoords (unsigned Num, double* x1, double* y1, double* x2, do
 			pCycle->GetAngles2D (this, &a1, &a2);
 			if (sin(a0 - a1) * sin (a0 - a2) > 0) {
 				double sign = sin (a0 - a1) > 0.0 ? 1.0 : -1.0;
-				double tanb = fabs (tan ((M_PI - a0 + a1) / 2)), cosa = cos (a0), sina = sin (a0);
+				double tanb = 0., cosa = cos (a0), sina = sin (a0);
+				if (m_Begin->GetZ () == 6 && !reinterpret_cast <Atom*> (m_Begin)->GetShowSymbol ())
+					tanb = fabs (tan ((M_PI - a0 + a1) / 2));
 				m_coords[4] = *x1 + BondDist * cosa * tanb - dy * sign;
 				m_coords[5] = *y1 + dx * sign - BondDist * sina * tanb;
-				tanb = fabs (tan ((a2 - a0) / 2));
+				tanb = (m_End->GetZ () == 6 && !reinterpret_cast <Atom*> (m_End)->GetShowSymbol ())? fabs (tan ((a2 - a0) / 2)): 0.;
 				m_coords[6] = *x2 - BondDist * cosa * tanb - dy * sign;
 				m_coords[7] = *y2 + dx * sign + BondDist * sina * tanb;
-			} else {
-				m_coords[0] = *x1 - dy / 2;
-				m_coords[1] = *y1 + dx / 2;
-				m_coords[2] = *x2 - dy / 2;
-				m_coords[3] = *y2 + dx / 2;
-				m_coords[4] = *x1 + dy / 2;
-				m_coords[5] = *y1 - dx / 2;
-				m_coords[6] = *x2 + dy / 2;
-				m_coords[7] = *y2 - dx / 2;
-			}
+			} else goto general;
 		} else {
+general:
+			// search how many bonds have each atom
+			int n1 = m_Begin->GetBondsNumber () - 1, n2 = m_End->GetBondsNumber () - 1;
+			if (n1 == 1) {
+				// put the second line on the bond side if any
+				map <gcu::Atom*, gcu::Bond*>::iterator it;
+				Bond *bond = reinterpret_cast <Bond*> (m_Begin->GetFirstBond (it));
+				if (bond == this)
+					bond = reinterpret_cast <Bond*> (m_Begin->GetNextBond (it));
+				double a0 = atan2 (*y1 - *y2, *x2 - *x1), a1 = bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_Begin)), a2, a;
+				if (fabs (fabs (a0 - a1) - M_PI) > 0.01) {
+					double sign = sin (a0 - a1) > 0.0 ? 1.0 : -1.0;
+					double tanb = ((m_Begin->GetZ () == 6)? fabs (tan ((M_PI - a0 + a1) / 2)): 0.), cosa = cos (a0), sina = sin (a0);
+					m_coords[4] = *x1 + BondDist * cosa * tanb - dy * sign;
+					m_coords[5] = *y1 + dx * sign - BondDist * sina * tanb;
+					tanb = 0.;
+					a2 = M_PI + a0;
+					if (a2 > 2 * M_PI)
+						a2 -= 2 * M_PI;
+					bond = reinterpret_cast <Bond*> (m_End->GetFirstBond (it));
+					if (m_End->GetZ () == 6)
+						while (bond) {
+							if (bond != this) {
+								a = tan ((bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_End)) - a0) / 2);
+								if (sign * a < sign * tanb)
+									tanb = a;
+								
+							}
+							bond = reinterpret_cast <Bond*> (m_End->GetNextBond (it));
+						}
+					m_coords[6] = *x2 - BondDist * cosa * tanb - dy * sign;
+					m_coords[7] = *y2 + dx * sign + BondDist * sina * tanb;
+					goto done;
+				}
+			} else if (n1 > 1 && n2 > 0) {
+				if (n2 == 1) {
+					map <gcu::Atom*, gcu::Bond*>::iterator it;
+					Bond *bond = reinterpret_cast <Bond*> (m_End->GetFirstBond (it));
+					if (bond == this)
+						bond = reinterpret_cast <Bond*> (m_End->GetNextBond (it));
+					double a0 = atan2 (*y1 - *y2, *x2 - *x1), a1 = bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_End)), a2, a;
+					if (fabs (fabs (a0 - a1) - M_PI) > 0.01) {
+						double sign = sin (a0 - a1) > 0.0 ? 1.0 : -1.0;
+						double tanb = 0., cosa = cos (a0), sina = sin (a0);
+						a2 = M_PI + a0;
+						if (a2 > 2 * M_PI)
+							a2 -= 2 * M_PI;
+						bond = reinterpret_cast <Bond*> (m_Begin->GetFirstBond (it));
+						if (m_Begin->GetZ () == 6)
+							while (bond) {
+								if (bond != this) {
+									a = tan ((bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_Begin)) - a2) / 2);
+									if (sign * a < sign * tanb)
+										tanb = a;
+									
+								}
+								bond = reinterpret_cast <Bond*> (m_Begin->GetNextBond (it));
+							}
+						m_coords[4] = *x1 + BondDist * cosa * tanb - dy * sign;
+						m_coords[5] = *y1 + dx * sign - BondDist * sina * tanb;
+						tanb = (m_End->GetZ () == 6)? fabs (tan ((a1 - a0) / 2)): 0.;
+						m_coords[6] = *x2 - BondDist * cosa * tanb - dy * sign;
+						m_coords[7] = *y2 + dx * sign + BondDist * sina * tanb;
+					}
+				} else {
+					// use the side with more room.
+					double tana = 0., tanb = 0., tanc = 0., tand = 0.;
+					double a, a0 = GetAngle2DRad (reinterpret_cast <Atom*> (m_End)), a1 = GetAngle2DRad (reinterpret_cast <Atom*> (m_Begin));
+					double cosa = cos (a0), sina = sin (a0);
+					map <gcu::Atom*, gcu::Bond*>::iterator it;
+					Bond *bond;
+					if (m_Begin->GetZ () == 6) {
+						bond = reinterpret_cast <Bond*> (m_Begin->GetFirstBond (it));
+						while (bond) {
+							if (bond != this) {
+								a = tan ((bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_Begin)) - a0) / 2);
+								if (a > tana)
+									tana = a;
+								if (a < tanb)
+									tanb = a;
+								
+							}
+							bond = reinterpret_cast <Bond*> (m_Begin->GetNextBond (it));
+						}
+					}
+					if (m_End->GetZ () == 6) {
+						bond = reinterpret_cast <Bond*> (m_End->GetFirstBond (it));
+						while (bond) {
+							if (bond != this) {
+								a = tan ((bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_End)) - a1) / 2);
+								if (a > tanc)
+									tanc = a;
+								if (a < tand)
+									tand = a;
+								
+							}
+							bond = reinterpret_cast <Bond*> (m_End->GetNextBond (it));
+						}
+					}
+					if (tana - tand > tanc - tanb) {
+						m_coords[4] = *x1 + BondDist * cosa * tanb + dy;
+						m_coords[5] = *y1 - dx - BondDist * sina * tanb;
+						m_coords[6] = *x2 + BondDist * cosa * tanc + dy;
+						m_coords[7] = *y2 - dx - BondDist * sina * tanc;
+					} else {
+						m_coords[4] = *x1 - BondDist * cosa * tana - dy;
+						m_coords[5] = *y1 + dx + BondDist * sina * tana;
+						m_coords[6] = *x2 - BondDist * cosa * tand - dy;
+						m_coords[7] = *y2 + dx + BondDist * sina * tand;
+					}
+				}
+				goto done;
+			} else if (n2 == 1) { // n1 is 0
+				map <gcu::Atom*, gcu::Bond*>::iterator it;
+				Bond *bond = reinterpret_cast <Bond*> (m_End->GetFirstBond (it));
+				if (bond == this)
+					bond = reinterpret_cast <Bond*> (m_End->GetNextBond (it));
+				double a0 = atan2 (*y1 - *y2, *x2 - *x1), a1 = bond->GetAngle2DRad (reinterpret_cast <Atom*> (m_End));
+				if (fabs (fabs (a0 - a1) - M_PI) > 0.01) {
+					double sign = sin (a0 - a1) > 0.0 ? 1.0 : -1.0;
+					double tanb, cosa = cos (a0), sina = sin (a0);
+					m_coords[4] = *x1 - dy * sign;
+					m_coords[5] = *y1 + dx * sign;
+					tanb = (m_End->GetZ () == 6)? fabs (tan ((a1 - a0) / 2)): 0.;
+					m_coords[6] = *x2 - BondDist * cosa * tanb - dy * sign;
+					m_coords[7] = *y2 + dx * sign + BondDist * sina * tanb;
+					goto done;
+				}
+			}
 			m_coords[0] = *x1 - dy / 2;
 			m_coords[1] = *y1 + dx / 2;
 			m_coords[2] = *x2 - dy / 2;
@@ -175,6 +331,7 @@ bool Bond::GetLine2DCoords (unsigned Num, double* x1, double* y1, double* x2, do
 		}
 		m_CoordsCalc = true;
 	}
+done:
 	Num--;
 	Num *= 4;
 	*x1 = m_coords[Num++];
@@ -196,7 +353,7 @@ Object* Bond::GetAtomAt(double x, double y, double z)
 	return NULL;
 }
 
-bool Bond::SaveNode (xmlDocPtr xml, xmlNodePtr node)
+bool Bond::SaveNode (xmlDocPtr xml, xmlNodePtr node) const
 {
 	switch(m_type) {
 	case UpBondType:
@@ -380,7 +537,7 @@ void Bond::SetSelected (GtkWidget* w, int state)
 	}
 }
 
-void Bond::Add (GtkWidget* w)
+void Bond::Add (GtkWidget* w) const
 {
 	if (!w)
 		return;
@@ -400,17 +557,17 @@ void Bond::Add (GtkWidget* w)
 	GnomeCanvasItem* item = NULL;
 	group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (pData->Group, gnome_canvas_group_ext_get_type (), NULL));
 	g_signal_connect (G_OBJECT (group), "event", G_CALLBACK (on_event), w);
-	g_object_set_data (G_OBJECT (group), "object", this);
+	g_object_set_data (G_OBJECT (group), "object", (void *) this);
 	bool result = false;
 	if (m_Crossing.size () > 0) {
-		map<Bond*, BondCrossing>::iterator i, iend = m_Crossing.end ();
+		map<Bond*, BondCrossing>::const_iterator i, iend = m_Crossing.end ();
 		for (i = m_Crossing.begin (); i != iend; i++)
 			if ((result |= (*i).second.is_before))
 				break;
 	}
 	GnomeCanvasPathDef *path = NULL;
 	if (result) {
-		path = BuildCrossingPathDef (pData);
+		path = const_cast <Bond *> (this)->BuildCrossingPathDef (pData);
 		if (path) {
 			GnomeCanvasItem *item = (m_type == NormalBondType || m_type == UndeterminedBondType)?
 				gnome_canvas_item_new (
@@ -428,7 +585,7 @@ void Bond::Add (GtkWidget* w)
 					"width_units", 0.,
 					NULL);
 			g_object_set_data (G_OBJECT (group), "back", item);
-			g_object_set_data (G_OBJECT (item), "object", this);
+			g_object_set_data (G_OBJECT (item), "object", (void *) this);
 			g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
 			/* now bring to front, FIXME: not secure ! */
 			gnome_canvas_item_lower_to_bottom (item);
@@ -442,7 +599,7 @@ void Bond::Add (GtkWidget* w)
 			gnome_canvas_path_def_unref (path);
 		}
 	}
-	path = BuildPathDef (pData);
+	path = const_cast <Bond *> (this)->BuildPathDef (pData);
 	switch (GetType ())
 	{
 		case NormalBondType:
@@ -469,7 +626,7 @@ void Bond::Add (GtkWidget* w)
 	}
 	gnome_canvas_path_def_unref (path);
 	g_object_set_data (G_OBJECT (group), "path", item);
-	g_object_set_data (G_OBJECT (item), "object", this);
+	g_object_set_data (G_OBJECT (item), "object", (void *) this);
 	g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
 	pData->Items[this] = group;
 	if (pAtom0->GetParent ()->GetType () == FragmentType)
@@ -497,24 +654,6 @@ void Bond::Add (GtkWidget* w)
 	}
 }
 
-bool Bond::ReplaceAtom (Atom* oldAtom, Atom* newAtom)
-{
-	if (oldAtom == m_Begin) {
-		if (m_End)
-			m_End->RemoveBond (this);
-		m_Begin = newAtom;
-		if (m_Begin && m_End)
-			m_End->AddBond (this);
-	} else if (oldAtom == m_End) {
-		if (m_Begin)
-			m_Begin->RemoveBond (this);
-		m_End = newAtom;
-		if (m_Begin && m_End)
-			m_Begin->AddBond (this);
-	}
-	return true;
-}
-
 GnomeCanvasPathDef* Bond::BuildPathDef (WidgetData* pData)
 {
 	double x1, y1, x2, y2, x, y, dx, dy, dx1, dy1;
@@ -680,7 +819,7 @@ GnomeCanvasPathDef *Bond::BuildCrossingPathDef (WidgetData* pData)
 	return path;
 }
 
-void Bond::Update(GtkWidget* w)
+void Bond::Update(GtkWidget* w) const
 {
 	if (!w || !m_order)
 		return;
@@ -690,7 +829,7 @@ void Bond::Update(GtkWidget* w)
 	Theme *pTheme = pData->m_View->GetDoc ()->GetTheme ();
 	bool result = false;
 	if (m_Crossing.size () > 0) {
-		map<Bond*, BondCrossing>::iterator i, iend = m_Crossing.end ();
+		map<Bond*, BondCrossing>::const_iterator i, iend = m_Crossing.end ();
 		for (i = m_Crossing.begin (); i != iend; i++)
 			if ((result |= (*i).second.is_before))
 				break;
@@ -704,7 +843,7 @@ void Bond::Update(GtkWidget* w)
 	}
 	obj = g_object_get_data(G_OBJECT(group), "back");
 	if (result) {
-		path = BuildCrossingPathDef (pData);
+		path = const_cast <Bond *> (this)->BuildCrossingPathDef (pData);
 		if (path) {
 			if (obj)
 				g_object_set (obj, "bpath", path, NULL);
@@ -725,7 +864,7 @@ void Bond::Update(GtkWidget* w)
 						"width_units", 0.,
 						NULL);
 				g_object_set_data (G_OBJECT (group), "back", item);
-				g_object_set_data (G_OBJECT (item), "object", this);
+				g_object_set_data (G_OBJECT (item), "object", (void *) this);
 				g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
 				/* now bring to front, FIXME: not secure ! */
 				gnome_canvas_item_lower_to_bottom (item);
@@ -742,7 +881,7 @@ void Bond::Update(GtkWidget* w)
 	} else if (obj) {
 		g_object_set_data (G_OBJECT(group), "back", NULL);
 	}
-	path = BuildPathDef (pData);
+	path = const_cast <Bond *> (this)->BuildPathDef (pData);
 	obj = g_object_get_data(G_OBJECT(group), "path");
 	g_object_set (obj, "bpath", path, NULL);
 	if (m_type == NormalBondType || m_type == UndeterminedBondType)
