@@ -4,7 +4,7 @@
  * GChemPaint library
  * fragment.cc 
  *
- * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -24,7 +24,7 @@
 
 #include "config.h"
 #include "fragment.h"
-#include "fragment-atom.h"
+#include "fragment-residue.h"
 #include "widgetdata.h"
 #include "document.h"
 #include "application.h"
@@ -38,6 +38,8 @@
 #include <canvas/gcp-canvas-rect-ellipse.h>
 #include <canvas/gcp-canvas-bpath.h>
 #include <gcu/element.h>
+#include <gcu/formula.h>
+#include <gcu/objprops.h>
 #include <pango/pango-attributes.h>
 #include <glib/gi18n-lib.h>
 #include <list>
@@ -59,7 +61,9 @@ static void on_fragment_sel_changed (Fragment *fragment, struct GnomeCanvasPango
 	fragment->OnSelChanged (bounds);
 }
 
-Fragment::Fragment (): TextObject (FragmentType)
+Fragment::Fragment ():
+	TextObject (FragmentType),
+	m_Inversable (false)
 {
 	m_Atom = new FragmentAtom (this, 0);
 	m_BeginAtom = m_EndAtom = 0;
@@ -69,7 +73,9 @@ Fragment::Fragment (): TextObject (FragmentType)
 	SetId ("f1");
 }
 
-Fragment::Fragment (double x, double y): TextObject (x, y, FragmentType)
+Fragment::Fragment (double x, double y):
+	TextObject (x, y, FragmentType),
+	m_Inversable (false)
 {
 	m_Atom = new FragmentAtom (this, 0);
 	m_Atom->SetCoords (x, y);
@@ -112,14 +118,48 @@ bool Fragment::OnChanged (bool save)
 		pango_layout_iter_free (iter);
 	}
 	/*main atom management*/
-	if (!m_Atom->GetZ ()) {
-		int Z = GetElementAtPos (m_StartSel, CurPos);
-		if (!Z && m_StartSel > m_BeginAtom)
-			Z = GetElementAtPos (m_StartSel = m_BeginAtom, CurPos);
-		if (Z) {
-			m_Atom->SetZ (Z);
+	FragmentResidue *residue = dynamic_cast <FragmentResidue*> (m_Atom);
+	Residue *r = NULL;
+	char sy[Residue::MaxSymbolLength + 1];
+	if (!m_Atom->GetSymbol ())
+		m_StartSel = m_BeginAtom;
+	if (!m_Atom->GetZ () || (residue != NULL && residue->GetResidue () == NULL)) {
+		strncpy (sy, m_buf.c_str () + m_StartSel, Residue::MaxSymbolLength);
+		int i = Residue::MaxSymbolLength;
+		while (i > 0) {
+			sy[i] = 0;
+			r = (Residue *) Residue::GetResidue (sy, NULL);
+			if (r)
+				break;
+			i--;
+		}
+		if (r) {
+			CurPos = m_StartSel + i;
 			m_BeginAtom = m_StartSel;
 			m_EndAtom = CurPos;
+			if (residue)
+				residue->SetResidue (r);
+			else {
+				map<gcu::Atom*, gcu::Bond*>::iterator i;
+				Bond *pBond = (gcp::Bond*) m_Atom->GetFirstBond (i);
+				Atom *pOldAtom = m_Atom;
+				m_Atom = new FragmentResidue (this, sy);
+				m_Atom->SetId ((gchar*) pOldAtom->GetId ());
+				if (pBond) {
+					pBond->ReplaceAtom (pOldAtom, m_Atom);
+					m_Atom->AddBond (pBond);
+				}
+				delete pOldAtom;
+			}
+		} else {
+			int Z = GetElementAtPos (m_StartSel, CurPos);
+			if (!Z && m_StartSel > m_BeginAtom)
+				Z = GetElementAtPos (m_StartSel = m_BeginAtom, CurPos);
+			if (Z) {
+				m_Atom->SetZ (Z);
+				m_BeginAtom = m_StartSel;
+				m_EndAtom = CurPos;
+			}
 		}
 	} else if (m_EndSel <= m_BeginAtom) {
 		int delta = CurPos - m_EndSel;
@@ -127,17 +167,44 @@ bool Fragment::OnChanged (bool save)
 		m_EndAtom += delta;
 	} else if ((m_EndAtom <= m_EndSel && m_EndAtom >= m_StartSel) ||
 		(m_BeginAtom <= m_EndSel && m_BeginAtom >= m_StartSel) ||
-		(m_BeginAtom + 3 >= CurPos)) {
+		(m_BeginAtom + Residue::MaxSymbolLength >= CurPos)) {
 		if (m_BeginAtom > m_StartSel)
 			m_BeginAtom = m_StartSel;
 		if (m_EndAtom > CurPos)
 			m_EndAtom = CurPos;
-		else if (m_EndAtom < m_BeginAtom + 3)
-			m_EndAtom = m_BeginAtom + 3;
-		int Z = GetElementAtPos (m_BeginAtom, m_EndAtom);
-		m_Atom->SetZ (Z);
-		if (!Z)
-			m_EndAtom = CurPos;
+		else if (m_EndAtom < m_BeginAtom + Residue::MaxSymbolLength)
+			m_EndAtom = m_BeginAtom + Residue::MaxSymbolLength;
+		strncpy (sy, m_buf.c_str () + m_BeginAtom, Residue::MaxSymbolLength);
+		int i = Residue::MaxSymbolLength;
+		while (i > 0) {
+			sy[i] = 0;
+			r = (Residue *) Residue::GetResidue (sy, NULL);
+			if (r)
+				break;
+			i--;
+		}
+		if (r) {
+			m_EndAtom = m_BeginAtom + i;
+			if (residue)
+				residue->SetResidue (r);
+			else {
+				map<gcu::Atom*, gcu::Bond*>::iterator i;
+				Bond *pBond = (gcp::Bond*) m_Atom->GetFirstBond (i);
+				Atom *pOldAtom = m_Atom;
+				m_Atom = new FragmentResidue (this, sy);
+				m_Atom->SetId ((gchar*) pOldAtom->GetId ());
+				if (pBond) {
+					pBond->ReplaceAtom (pOldAtom, m_Atom);
+					m_Atom->AddBond (pBond);
+				}
+				delete pOldAtom;
+			}
+		} else {
+			int Z = GetElementAtPos (m_BeginAtom, m_EndAtom);
+			m_Atom->SetZ (Z);
+			if (!Z)
+				m_EndAtom = CurPos;
+		}
 	}
 	PangoRectangle rect;
 	pango_layout_index_to_pos (m_Layout, m_BeginAtom, &rect);
@@ -181,7 +248,7 @@ bool Fragment::OnChanged (bool save)
 	return true;
 }
 
-void Fragment::Add (GtkWidget* w)
+void Fragment::Add (GtkWidget* w) const
 {
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
 	if (pData->Items[this] != NULL)
@@ -190,38 +257,38 @@ void Fragment::Add (GtkWidget* w)
 	Theme *pTheme = pView->GetDoc ()->GetTheme ();
 	if (m_ascent <= 0) {
 		PangoContext* pc = pView->GetPangoContext ();
-		m_Layout = pango_layout_new (pc);
+		const_cast <Fragment *> (this)->m_Layout = pango_layout_new (pc);
 		PangoAttrList *l = pango_attr_list_new ();
 		pango_layout_set_attributes (m_Layout, l);
 		pango_layout_set_font_description (m_Layout, pView->GetPangoFontDesc ());
 		pango_layout_set_text (m_Layout, "l", 1);
 		PangoLayoutIter* iter = pango_layout_get_iter (m_Layout);
-		m_ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
+		const_cast <Fragment *> (this)->m_ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
 		pango_layout_iter_free (iter);
 		pango_layout_set_text (m_Layout, "C", 1);
 		PangoRectangle rect;
 		pango_layout_get_extents (m_Layout, &rect, NULL);
-		m_CHeight =  double (rect.height / PANGO_SCALE) / 2.0;
+		const_cast <Fragment *> (this)->m_CHeight =  double (rect.height / PANGO_SCALE) / 2.0;
 		pango_layout_set_text (m_Layout, m_buf.c_str (), -1);
 		if (m_AttrList) {
 			pango_layout_set_attributes (m_Layout, m_AttrList);
 			pango_attr_list_unref (m_AttrList);
-			m_AttrList = NULL;
+			const_cast <Fragment *> (this)->m_AttrList = NULL;
 		}
 		if (m_buf.length () > 0) {
-			m_buf.clear ();
+			const_cast <Fragment *> (this)->m_buf.clear ();
 			pango_layout_index_to_pos (m_Layout, m_BeginAtom, &rect);
-			m_lbearing = rect.x / PANGO_SCALE;
+			const_cast <Fragment *> (this)->m_lbearing = rect.x / PANGO_SCALE;
 			pango_layout_index_to_pos (m_Layout, m_EndAtom, &rect);
-			m_lbearing += rect.x / PANGO_SCALE;
-			m_lbearing /=  2;
+			const_cast <Fragment *> (this)->m_lbearing += rect.x / PANGO_SCALE;
+			const_cast <Fragment *> (this)->m_lbearing /=  2;
 			iter = pango_layout_get_iter (m_Layout);
-			m_ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
+			const_cast <Fragment *> (this)->m_ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
 			pango_layout_iter_free (iter);
 		}
 		pango_layout_get_extents (m_Layout, NULL, &rect);
-		m_length = rect.width / PANGO_SCALE;
-		m_height = rect.height / PANGO_SCALE;
+		const_cast <Fragment *> (this)->m_length = rect.width / PANGO_SCALE;
+		const_cast <Fragment *> (this)->m_height = rect.height / PANGO_SCALE;
 	}
 	GnomeCanvasGroup* group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (pData->Group, gnome_canvas_group_ext_get_type (), NULL)), *chgp;
 	GnomeCanvasItem* item = gnome_canvas_item_new(
@@ -231,12 +298,11 @@ void Fragment::Add (GtkWidget* w)
 						"y1", m_y * pTheme->GetZoomFactor () - pTheme->GetPadding () - m_ascent + m_CHeight,
 						"x2", m_x * pTheme->GetZoomFactor () + m_length + pTheme->GetPadding () - m_lbearing,
 						"y2", m_y * pTheme->GetZoomFactor () + m_height + pTheme->GetPadding () - m_ascent + m_CHeight,
-						"fill_color", "white",
-						"outline_color", "white",
 						NULL);
+	gnome_canvas_item_hide (item);
 	g_object_set_data (G_OBJECT (group), "rect", item);
 	g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
-	g_object_set_data (G_OBJECT (item), "object", this);
+	g_object_set_data (G_OBJECT (item), "object", (void *) this);
 	item = gnome_canvas_item_new (
 						group,
 						gnome_canvas_pango_get_type (),
@@ -246,16 +312,16 @@ void Fragment::Add (GtkWidget* w)
 						"editing", false,
 						NULL);
 	g_object_set_data (G_OBJECT (group), "fragment", item);
-	g_object_set_data (G_OBJECT (item), "object", this);
+	g_object_set_data (G_OBJECT (item), "object", (void *) this);
 	g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
-	g_signal_connect_swapped (G_OBJECT (item), "changed", G_CALLBACK (on_fragment_changed), this);
-	g_signal_connect_swapped (G_OBJECT (item), "sel-changed", G_CALLBACK (on_fragment_sel_changed), this);
+	g_signal_connect_swapped (G_OBJECT (item), "changed", G_CALLBACK (on_fragment_changed), (void *) this);
+	g_signal_connect_swapped (G_OBJECT (item), "sel-changed", G_CALLBACK (on_fragment_sel_changed), (void *) this);
 	/* add charge */
 	int charge = m_Atom->GetCharge ();
 	if (charge) {
 		double x, y, Angle, Dist;
 		unsigned char Pos = m_Atom->Atom::GetChargePosition (&Angle, &Dist);
-		int align = GetChargePosition(m_Atom, Pos, 0., x, y);
+		int align = const_cast <Fragment *> (this)->GetChargePosition (m_Atom, Pos, 0., x, y);
 		if (Dist != 0.) {
 			x = m_x + Dist * cos (Angle);
 			y = m_y - Dist * sin (Angle);
@@ -328,6 +394,7 @@ void Fragment::Add (GtkWidget* w)
 		g_object_set_data (G_OBJECT (group), "sign", item);
 	}
 	pData->Items[this] = group;
+	const_cast <FragmentAtom *> (m_Atom)->DoBuildSymbolGeometry (pData->m_View);
 }
 
 void Fragment::SetSelected (GtkWidget* w, int state)
@@ -335,10 +402,12 @@ void Fragment::SetSelected (GtkWidget* w, int state)
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
 	GnomeCanvasGroup* group = pData->Items[this];
 	gchar const *chargecolor, *color;
+	bool visible = true;
 	switch (state) {	
 	case SelStateUnselected:
-		color = "white";
+		color = NULL;
 		chargecolor = "black";
+		visible = false;
 		break;
 	case SelStateSelected:
 		chargecolor = color = SelectColor;
@@ -350,20 +419,25 @@ void Fragment::SetSelected (GtkWidget* w, int state)
 		chargecolor = color = DeleteColor;
 		break;
 	default:
-		color = "white";
+		color = NULL;
 		chargecolor = "black";
 		break;
 	}
-	g_object_set (G_OBJECT (g_object_get_data (G_OBJECT (group), "rect")),
-				"fill_color", color, NULL);
 	gpointer item;
+	item = g_object_get_data (G_OBJECT (group), "rect");
+	g_object_set (G_OBJECT (item),
+				"fill_color", color, NULL);
+	if (visible)
+		gnome_canvas_item_show (GNOME_CANVAS_ITEM (item));
+	else
+		gnome_canvas_item_hide (GNOME_CANVAS_ITEM (item));
 	if ((item = g_object_get_data (G_OBJECT (group), "circle")))
 		g_object_set (item, "outline_color", chargecolor, NULL);
 	if ((item = g_object_get_data (G_OBJECT (group), "sign")))
 		g_object_set (item, "outline_color", chargecolor, NULL);
 }
 
-void Fragment::Update (GtkWidget* w)
+void Fragment::Update (GtkWidget* w) const
 {
 	WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (w), "data");
 	Theme *pTheme = pData->m_View->GetDoc ()->GetTheme ();
@@ -386,7 +460,7 @@ void Fragment::Update (GtkWidget* w)
 		double x, y, Angle, Dist;
 		unsigned char Pos = m_Atom->Atom::GetChargePosition (&Angle, &Dist);
 		if (item) {
-			int align = GetChargePosition (m_Atom, Pos, Angle, x, y);
+			int align = const_cast <Fragment *> (this)->GetChargePosition (m_Atom, Pos, Angle, x, y);
 			if (Dist != 0.) {
 				x = m_x + Dist * cos (Angle);
 				y = m_y - Dist * sin (Angle);
@@ -445,7 +519,7 @@ void Fragment::Update (GtkWidget* w)
 			gnome_canvas_path_def_unref (cpd);
 		} else {
 			GnomeCanvasGroup *chgp;
-			int align = GetChargePosition (m_Atom, Pos, Angle, x, y);
+			int align = const_cast <Fragment *> (this)->GetChargePosition (m_Atom, Pos, Angle, x, y);
 			x *= pTheme->GetZoomFactor ();
 			if (Dist != 0.) {
 				x = m_x + Dist * cos (Angle);
@@ -523,12 +597,13 @@ void Fragment::Update (GtkWidget* w)
 			g_object_set_data ((GObject*) group, "charge", NULL);
 		}
 	}
+	const_cast <FragmentAtom *> (m_Atom)->DoBuildSymbolGeometry (pData->m_View);
 }
 
-xmlNodePtr Fragment::Save (xmlDocPtr xml)
+xmlNodePtr Fragment::Save (xmlDocPtr xml) const
 {
-	m_buf = pango_layout_get_text (m_Layout);
-	if (m_RealSave && !Validate ())
+	const_cast <Fragment *> (this)->m_buf = pango_layout_get_text (m_Layout);
+	if (m_RealSave && !const_cast <Fragment *> (this)->Validate ())
 		return NULL;
 	xmlNodePtr node = xmlNewDocNode (xml, NULL, (xmlChar*) "fragment", NULL);
 	if (m_buf.length ()) {
@@ -574,7 +649,7 @@ bool filter_func (PangoAttribute *attribute, struct FilterStruct *s)
 	return FALSE;
 }
 
-bool Fragment::SavePortion (xmlDocPtr xml, xmlNodePtr node, unsigned start, unsigned end)
+bool Fragment::SavePortion (xmlDocPtr xml, xmlNodePtr node, unsigned start, unsigned end) const
 {
 	xmlNodePtr child;
 	struct FilterStruct s;
@@ -582,7 +657,7 @@ bool Fragment::SavePortion (xmlDocPtr xml, xmlNodePtr node, unsigned start, unsi
 	s.end = end;
 	int charge;
 	if (m_AttrList == NULL)
-		m_AttrList = pango_layout_get_attributes (m_Layout);
+		const_cast <Fragment *> (this)->m_AttrList = pango_layout_get_attributes (m_Layout);
 	pango_attr_list_filter (m_AttrList, (PangoAttrFilterFunc) filter_func, &s);
 	list<PangoAttribute*>::iterator i, iend = s.pal.end ();
 	string str;
@@ -631,7 +706,7 @@ bool Fragment::SavePortion (xmlDocPtr xml, xmlNodePtr node, unsigned start, unsi
 	return true;
 }
 
-xmlNodePtr Fragment::SaveSelection (xmlDocPtr xml)
+xmlNodePtr Fragment::SaveSelection (xmlDocPtr xml) const
 {
 	xmlNodePtr node = xmlNewDocNode (xml, NULL, (xmlChar*) "fragment", NULL);
 	if (!node)
@@ -667,6 +742,24 @@ bool Fragment::Load (xmlNodePtr node)
 			m_buf += m_Atom->GetSymbol();
 			m_Atom->SetCoords (m_x, m_y);
 			m_EndAtom = m_buf.length ();
+		} else if (!strcmp ((const char*) child->name, "residue")) {
+			// replace the atom by a residue
+			map<gcu::Atom*, gcu::Bond*>::iterator i;
+			Bond *pBond = (gcp::Bond*) m_Atom->GetFirstBond (i);
+			Atom *pOldAtom = m_Atom;
+			m_Atom = new FragmentResidue (this, NULL);
+			if (pBond) {
+				pBond->ReplaceAtom (pOldAtom, m_Atom);
+				m_Atom->AddBond (pBond);
+			}
+			delete pOldAtom;
+			AddChild (m_Atom);
+			if (!m_Atom->Load (child))
+				return false;
+			m_BeginAtom = m_buf.length ();
+			m_buf += m_Atom->GetSymbol ();
+			m_Atom->SetCoords (m_x, m_y);
+			m_EndAtom = m_buf.length ();
 		} else if (!strcmp ((const char*) child->name, "charge")) {
 			int start = m_buf.length (), end;
 			tmp = (char*) xmlGetProp (child, (xmlChar*) "value");
@@ -942,7 +1035,7 @@ int Fragment::GetElementAtPos (unsigned start, unsigned &end)
 	int Z;
 	char text[4];
 	memset (text, 0, 4);
-	strncpy (text, pango_layout_get_text (m_Layout) + start, 3);
+	strncpy (text, ((m_Layout)?pango_layout_get_text ( m_Layout): m_buf.c_str ()) + start, 3);
 	for (unsigned i = strlen (text); i > 0; i--) {
 		text[i] = 0;
 		if ((Z = Element::Z (text))) {
@@ -996,33 +1089,33 @@ int Fragment::GetChargePosition (FragmentAtom *pAtom, unsigned char &Pos, double
 		map<gcu::Atom*, gcu::Bond*>::iterator i;
 		Bond* pBond = (Bond*)m_Atom->GetFirstBond(i);
 		double angle = pBond->GetAngle2D(m_Atom) + 180.0;
-		if ((result & CHARGE_NE) && (angle >= 180.0) && (angle <= 270.0)) result -= CHARGE_NE;
-		if ((result & CHARGE_NW) && (((angle >= 270.0) && (angle <= 360.0)) || (fabs(angle) < 0.1))) result -= CHARGE_NW;
-		if ((result & CHARGE_N) && (angle >= 225.0) && (angle <= 315.0)) result -= CHARGE_N;
-		if ((result & CHARGE_SE) && (angle >= 90.0) && (angle <= 180.0)) result -= CHARGE_SE;
-		if ((result & CHARGE_SW) && (((angle >= 0.0) && (angle <= 90.0)) || (fabs(angle - 360.0) < 0.1))) result -= CHARGE_SW;
-		if ((result & CHARGE_S) && (angle >= 45.0) && (angle <= 135.0)) result -= CHARGE_S;
-		if ((result & CHARGE_E) && ((angle <= 225.0) && (angle >= 135.0))) result -= CHARGE_E;
-		if ((result & CHARGE_W) && ((angle >= 315.0) || (angle <= 45.0))) result -= CHARGE_W;
+		if ((result & POSITION_NE) && (angle >= 180.0) && (angle <= 270.0)) result -= POSITION_NE;
+		if ((result & POSITION_NW) && (((angle >= 270.0) && (angle <= 360.0)) || (fabs(angle) < 0.1))) result -= POSITION_NW;
+		if ((result & POSITION_N) && (angle >= 225.0) && (angle <= 315.0)) result -= POSITION_N;
+		if ((result & POSITION_SE) && (angle >= 90.0) && (angle <= 180.0)) result -= POSITION_SE;
+		if ((result & POSITION_SW) && (((angle >= 0.0) && (angle <= 90.0)) || (fabs(angle - 360.0) < 0.1))) result -= POSITION_SW;
+		if ((result & POSITION_S) && (angle >= 45.0) && (angle <= 135.0)) result -= POSITION_S;
+		if ((result & POSITION_E) && ((angle <= 225.0) && (angle >= 135.0))) result -= POSITION_E;
+		if ((result & POSITION_W) && ((angle >= 315.0) || (angle <= 45.0))) result -= POSITION_W;
 	}
 	if (Pos == 0xff) {
 		if (result) {
-			if (result & CHARGE_NE)
-				Pos = CHARGE_NE;
-			else if (result & CHARGE_NW)
-				Pos = CHARGE_NW;
-			else if (result & CHARGE_N)
-				Pos = CHARGE_N;
-			else if (result & CHARGE_SE)
-				Pos = CHARGE_SE;
-			else if (result & CHARGE_SW)
-				Pos = CHARGE_SW;
-			else if (result & CHARGE_S)
-				Pos = CHARGE_S;
-			else if (result & CHARGE_E)
-				Pos = CHARGE_E;
-			else if (result & CHARGE_W)
-				Pos = CHARGE_W;
+			if (result & POSITION_NE)
+				Pos = POSITION_NE;
+			else if (result & POSITION_NW)
+				Pos = POSITION_NW;
+			else if (result & POSITION_N)
+				Pos = POSITION_N;
+			else if (result & POSITION_SE)
+				Pos = POSITION_SE;
+			else if (result & POSITION_SW)
+				Pos = POSITION_SW;
+			else if (result & POSITION_S)
+				Pos = POSITION_S;
+			else if (result & POSITION_E)
+				Pos = POSITION_E;
+			else if (result & POSITION_W)
+				Pos = POSITION_W;
 		} else
 			return 0;
 	} else if (Pos) {
@@ -1032,35 +1125,35 @@ int Fragment::GetChargePosition (FragmentAtom *pAtom, unsigned char &Pos, double
 		return 0;
 
 	switch (Pos) {
-	case CHARGE_NE:
+	case POSITION_NE:
 		x = m_x + width / 2.0;
 		y = m_y - height / 2.0;
 		return 1;
-	case  CHARGE_NW:
+	case  POSITION_NW:
 		x = m_x - width / 2.0;
 		y = m_y - height / 2.0;
 		return -1;
-	case  CHARGE_N:
+	case  POSITION_N:
 		x = m_x;
 		y = m_y - height / 2.0;
 		return 2;
-	case CHARGE_SE:
+	case POSITION_SE:
 		x = m_x + width / 2.0;
 		y = m_y + height / 2.0;
 		return 1;
-	case CHARGE_SW:
+	case POSITION_SW:
 		x = m_x - width / 2.0;
 		y = m_y + height / 2.0;
 		return -1;
-	case CHARGE_S:
+	case POSITION_S:
 		x = m_x;
 		y = m_y + height / 2.0;
 		return -2;
-	case CHARGE_E:
+	case POSITION_E:
 		x = m_x + width / 2.0;
 		y = m_y;
 		return 1;
-	case CHARGE_W:
+	case POSITION_W:
 		x = m_x - width / 2.0;
 		y = m_y;
 		return -1;
@@ -1075,7 +1168,7 @@ bool Fragment::Validate ()
 	if ((m_buf.length () == 0)
 		&& m_Atom->GetBondsNumber () == 0)
 		return true;
-	if (m_Atom->GetZ() == 0) {
+	if (m_Atom->GetZ() == 0 || (dynamic_cast <FragmentResidue*> (m_Atom) && !((FragmentResidue*) m_Atom)->GetResidue ())) {
 		Document *pDoc = dynamic_cast<Document*> (GetDocument ());
 		GtkWidget* pWidget = pDoc->GetView ()->GetWidget ();
 		WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (pWidget), "data");
@@ -1147,4 +1240,145 @@ bool Fragment::GetPosition (double angle, double& x, double& y)
 	return false;
 }
 
+bool Fragment::SetProperty (unsigned property, char const *value)
+{
+	m_bLoading = true;
+	switch (property) {
+	case GCU_PROP_POS2D: {
+		sscanf (value, "%lg %lg", &m_x, &m_y);
+		gcu::Document *doc = GetDocument ();
+		if (doc) {
+			m_x *= doc->GetScale ();
+			m_y *= doc->GetScale ();
+		}
+		m_Atom->SetCoords (m_x, m_y);
+		break;
+	}
+	case GCU_PROP_TEXT_TEXT: {
+		m_buf = value;
+		if (m_EndAtom > m_BeginAtom) {
+			Residue *r = NULL;
+			char sy[Residue::MaxSymbolLength + 1];
+			strncpy (sy, m_buf.c_str () + m_BeginAtom, Residue::MaxSymbolLength);
+			int i = Residue::MaxSymbolLength;
+			while (i > 0) {
+				sy[i] = 0;
+				r = (Residue *) Residue::GetResidue (sy, NULL);
+				if (r)
+					break;
+				i--;
+			}
+			if (r) {
+				m_EndAtom = m_BeginAtom +  + strlen (sy);
+				map<gcu::Atom*, gcu::Bond*>::iterator i;
+				Bond *pBond = (gcp::Bond*) m_Atom->GetFirstBond (i);
+				Atom *pOldAtom = m_Atom;
+				pOldAtom->SetParent (NULL);
+				m_Atom = new FragmentResidue (this, sy);
+				AddChild (m_Atom);
+				m_Atom->SetId ((gchar*) pOldAtom->GetId ());
+				m_Atom->SetCoords (m_x, m_y);
+				if (pBond) {
+					pBond->ReplaceAtom (pOldAtom, m_Atom);
+					m_Atom->AddBond (pBond);
+				}
+				delete pOldAtom;
+			} else {
+				int Z = GetElementAtPos (m_BeginAtom, m_EndAtom);
+				if (Z)
+					m_Atom->SetZ (Z);
+			}
+			Analyze ();
+		}
+		break;
+	}
+	case GCU_PROP_FRAGMENT_ATOM_START:
+		m_BeginAtom = atoi (value);
+		m_EndAtom = m_BeginAtom + Residue::MaxSymbolLength;
+		if (m_buf.length ()) {
+			Residue *r = NULL;
+			char sy[Residue::MaxSymbolLength + 1];
+			strncpy (sy, m_buf.c_str () + m_BeginAtom, Residue::MaxSymbolLength);
+			int i = Residue::MaxSymbolLength;
+			while (i > 0) {
+				sy[i] = 0;
+				r = (Residue *) Residue::GetResidue (sy, NULL);
+				if (r)
+					break;
+				i--;
+			}
+			if (r) {
+				m_EndAtom = m_BeginAtom + strlen (sy);
+				map<gcu::Atom*, gcu::Bond*>::iterator i;
+				Bond *pBond = (gcp::Bond*) m_Atom->GetFirstBond (i);
+				Atom *pOldAtom = m_Atom;
+				pOldAtom->SetParent (NULL);
+				m_Atom = new FragmentResidue (this, sy);
+				AddChild (m_Atom);
+				m_Atom->SetId ((gchar*) pOldAtom->GetId ());
+				m_Atom->SetCoords (m_x, m_y);
+				if (pBond) {
+					pBond->ReplaceAtom (pOldAtom, m_Atom);
+					m_Atom->AddBond (pBond);
+				}
+				delete pOldAtom;
+			} else {
+				int Z = GetElementAtPos (m_BeginAtom, m_EndAtom);
+				if (Z)
+					m_Atom->SetZ (Z);
+			}
+			Analyze ();
+		}
+		break;
+	case GCU_PROP_FRAGMENT_ATOM_ID:
+		m_Atom->SetId (value);
+		if (!m_Atom->GetParent ())
+			AddChild (m_Atom);
+		break;
+	default:
+		break;
+	}
+	m_bLoading = false;
+	return true;
+}
+
+bool Fragment::Analyze () {
+	// search if main atom is at start or at end
+	if ((m_BeginAtom == 0 || m_EndAtom == m_buf.size ()) && ((m_EndAtom - m_BeginAtom) < m_buf.length ()) ){
+		m_Inversable = true;
+	}
+//	int valence = m_Atom->GetValence ();
+	if (m_AttrList != NULL)
+		pango_attr_list_unref (m_AttrList);
+	m_AttrList = pango_attr_list_new ();
+	AnalContent ();
+	return true;
+}
+
+void Fragment::Update () {
+	if (m_Atom->GetBondsNumber () > 0 && m_Inversable) {
+		map<gcu::Atom*, gcu::Bond*>::iterator i;
+		Bond *bond = reinterpret_cast <Bond *> (m_Atom->GetFirstBond (i));
+		double angle = bond->GetAngle2D (m_Atom);
+		if (m_BeginAtom == 0 && (angle < 89. && angle > -89.)) {
+			// build the formula, then write elements in reverse order might be unsecure in some cases (if linked atom has a stoichiometric coefficient)
+			Formula *formula = new Formula (m_buf, GCU_FORMULA_PARSE_RESIDUE);
+			std::list<FormulaElt *> const &elts = formula->GetElements ();
+			m_buf.clear ();
+			std::list<FormulaElt *>::const_reverse_iterator i, end = elts.rend ();
+			for (i = elts.rbegin (); i!= end; i++) {
+				m_buf += (*i)->Text ();
+			}
+			delete formula;
+			m_EndAtom = m_buf.length ();
+			m_BeginAtom = m_EndAtom - strlen (m_Atom->GetSymbol ());
+			if (m_AttrList != NULL)
+				pango_attr_list_unref (m_AttrList);
+			m_AttrList = pango_attr_list_new ();
+			AnalContent ();
+		} else if (angle > 91. || angle < -91.) {
+		}
+	}
+}
+
 }	//	namespace gcp
