@@ -4,7 +4,7 @@
  * GChemPaint library
  * view.cc
  *
- * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -37,8 +37,6 @@
 #include <canvas/gcp-canvas-group.h>
 #include <canvas/gcp-canvas-rect-ellipse.h>
 #include <gsf/gsf-output-gio.h>
-#include <libgnomevfs/gnome-vfs-file-info.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
 #include <cairo-pdf.h>
 #include <cairo-ps.h>
 #include <pango/pango-context.h>
@@ -61,11 +59,11 @@ typedef struct _GnomeCanvasGCP           GnomeCanvasGCP;
 typedef struct _GnomeCanvasGCPClass      GnomeCanvasGCPClass;
 
 #define GNOME_TYPE_CANVAS_GCP            (gnome_canvas_gcp_get_type ())
-#define GNOME_CANVAS_GCP(obj)            (GTK_CHECK_CAST ((obj), GNOME_TYPE_CANVAS_GCP, GnomeCanvasGCP))
-#define GNOME_CANVAS_CLASS_GCP(klass)    (GTK_CHECK_CLASS_CAST ((klass), GNOME_TYPE_CANVAS_GCP, GnomeCanvasGCPClass))
-#define GNOME_IS_CANVAS_GCP(obj)         (GTK_CHECK_TYPE ((obj), GNOME_TYPE_CANVAS_GCP))
-#define GNOME_IS_CANVAS_GCP_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GNOME_TYPE_CANVAS_GCP))
-#define GNOME_CANVAS_GCP_GET_CLASS(obj)  (GTK_CHECK_GET_CLASS ((obj), GNOME_TYPE_CANVAS_GCP, GnomeCanvasGCPClass))
+#define GNOME_CANVAS_GCP(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), GNOME_TYPE_CANVAS_GCP, GnomeCanvasGCP))
+#define GNOME_CANVAS_CLASS_GCP(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), GNOME_TYPE_CANVAS_GCP, GnomeCanvasGCPClass))
+#define GNOME_IS_CANVAS_GCP(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GNOME_TYPE_CANVAS_GCP))
+#define GNOME_IS_CANVAS_GCP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), GNOME_TYPE_CANVAS_GCP))
+#define GNOME_CANVAS_GCP_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), GNOME_TYPE_CANVAS_GCP, GnomeCanvasGCPClass))
 
 enum {
   UPDATE_BOUNDS,
@@ -149,6 +147,10 @@ void gnome_canvas_gcp_update_bounds (GnomeCanvasGCP *canvas)
 	gcp::WidgetData* pData = (gcp::WidgetData*) g_object_get_data (G_OBJECT (canvas), "data");
 	double x1, y1, x2, y2;
 	gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (pData->Group), &x1, &y1, &x2, &y2);
+	if (x1 > x2)
+		x1 = x2 = 0.;
+	if (y1 > y2)
+		y1 = y2 = 0.;
 	gcp::View *pView = (gcp::View*) g_object_get_data (G_OBJECT (canvas), "view");
 	pView->UpdateSize (x1, y1, x2, y2);
 }
@@ -244,7 +246,7 @@ bool View::OnEvent (GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 	x = event->button.x;
 	y = event->button.y;
 	gnome_canvas_item_w2i (GNOME_CANVAS_ITEM (m_pData->Group), &x, &y);
-	if (event->type == GDK_BUTTON_PRESS) {
+//	if (event->type == GDK_BUTTON_PRESS) {
 		if (item == m_pData->Background) {
 			item = NULL;
 			std::map<Object const*, GnomeCanvasGroup*>::iterator i = m_pData->Items.begin (),
@@ -257,12 +259,36 @@ bool View::OnEvent (GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 						item = GNOME_CANVAS_ITEM ((*i).second);
 						m_CurObject = pBond;
 						break;
+					} else {
+						// may be one of the atoms might work
+						gcu::Atom *pAtom = pBond->GetAtom (0);
+						double xa, ya;
+						pAtom->GetCoords (&xa, &ya, NULL);
+						xa *= pTheme->GetZoomFactor ();
+						ya *= pTheme->GetZoomFactor ();
+						xa =- x;
+						ya -= y;
+						if (sqrt (xa * xa + ya * ya) < 3.5) {
+							//3.5 is arbitrary
+							m_CurObject = pAtom;
+							break;
+						}
+						pAtom = pBond->GetAtom (1);
+						pAtom->GetCoords (&xa, &ya, NULL);
+						xa *= pTheme->GetZoomFactor ();
+						ya *= pTheme->GetZoomFactor ();
+						xa =- x;
+						ya -= y;
+						if (sqrt (xa * xa + ya * ya) < 3.5) {
+							m_CurObject = pAtom;
+							break;
+						}
 					}
 				}
 				i++;
 			}
 		}
-	}
+//	}
 	Object *pAtom;
 	if (m_CurObject && ((pAtom = m_CurObject->GetAtomAt (x / pTheme->GetZoomFactor (), y / pTheme->GetZoomFactor ()))))
 			m_CurObject = pAtom;
@@ -666,6 +692,13 @@ static void do_set_symbol (GtkAction *action, Object *obj)
 		Operation *op = pDoc->GetNewOperation (GCP_MODIFY_OPERATION);
 		op->AddObject (group);
 		atom->SetZ (Z);
+		// set all bonds as dirty
+		map<gcu::Atom*, gcu::Bond*>::iterator i;
+		Bond *bond = reinterpret_cast <Bond *> (atom->GetFirstBond (i));
+		while (bond) {
+			bond->SetDirty ();
+			bond = reinterpret_cast <Bond *> (atom->GetNextBond (i));
+		}
 		pDoc->GetView ()->Update (obj);
 		op->AddObject (group, 1);
 		pDoc->FinishOperation ();
@@ -808,6 +841,13 @@ bool View::OnKeyPress (GtkWidget* w, GdkEventKey* event)
 					op->AddObject (group);
 					atom->SetZ (Z);
 					Update (atom);
+					// set all bonds as dirty
+					map<gcu::Atom*, gcu::Bond*>::iterator i;
+					Bond *bond = reinterpret_cast <Bond *> (atom->GetFirstBond (i));
+					while (bond) {
+						bond->SetDirty ();
+						bond = reinterpret_cast <Bond *> (atom->GetNextBond (i));
+					}
 					op->AddObject (group, 1);
 					m_pDoc->FinishOperation ();
 				}
@@ -943,16 +983,11 @@ void View::OnSelectAll ()
 
 static gboolean do_save_image (const gchar *buf, gsize count, GError **error, gpointer data)
 {
-	GnomeVFSHandle *handle = (GnomeVFSHandle*) data;
-	GnomeVFSFileSize written = 0;
-	GnomeVFSResult res;
+	GOutputStream *output = (GOutputStream *) data;
 	while (count) {
-		res = gnome_vfs_write (handle, buf, count, &written);
-		if (res != GNOME_VFS_OK) {
-			g_set_error (error, g_quark_from_static_string ("gchempaint"), res, gnome_vfs_result_to_string (res));
+		count -= g_output_stream_write (output, buf, count, NULL, error);
+		if (*error)
 			return false;
-		}
-		count -= written;
 	}
 	return true;
 }
@@ -986,7 +1021,6 @@ void View::ExportImage (string const &filename, const char* type, int resolution
 		}
 		ArtDRect rect;
 		m_pData->GetObjectBounds (m_pDoc, &rect);
-		m_pData->ShowSelection (false);
 		cairo_surface_t *surface = NULL;
 		if (!strcmp (type, "pdf"))
 			surface = cairo_pdf_surface_create_for_stream (cairo_write_func, output, w, h);
@@ -1008,16 +1042,16 @@ void View::ExportImage (string const &filename, const char* type, int resolution
 		xmlFreeDoc (doc);
 	} else {
 		GdkPixbuf *pixbuf = BuildPixbuf (resolution);
-		GnomeVFSHandle *handle = NULL;
-		if (gnome_vfs_create (&handle, filename.c_str (), GNOME_VFS_OPEN_WRITE, true, 0644) == GNOME_VFS_OK) {
-			GError *error = NULL;
-			gdk_pixbuf_save_to_callbackv (pixbuf, do_save_image, handle, type, NULL, NULL, &error);
-			if (error) {
-				cerr << _("Unable to save image file: ") << error->message << endl;
-				g_error_free (error);
-			}
-			gnome_vfs_close (handle); // hope there will be no error there
+		GFile *file = g_vfs_get_file_for_uri (g_vfs_get_default (), filename.c_str ());
+		GError *error = NULL;
+		GFileOutputStream *output = g_file_create (file, G_FILE_CREATE_NONE, NULL, &error);
+		if (!error)
+			gdk_pixbuf_save_to_callbackv (pixbuf, do_save_image, output, type, NULL, NULL, &error);
+		if (error) {
+			fprintf (stderr, _("Unable to save image file: %s\n"), error->message);
+			g_error_free (error);
 		}
+		g_object_unref (file);
 		g_object_unref (pixbuf);
 	}
 	m_pData->ShowSelection (true);
